@@ -19,7 +19,6 @@
  * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
-#include "config.h"
 #include "px-backend-config.h"
 
 #include <glib.h>
@@ -101,8 +100,6 @@ struct _PxManager {
 
 G_DEFINE_TYPE (PxManager, px_manager, G_TYPE_OBJECT)
 
-G_DEFINE_QUARK (px - manager - error - quark, px_manager_error)
-
 static void
 px_manager_on_network_changed (GNetworkMonitor *monitor,
                                gboolean         network_available,
@@ -215,7 +212,10 @@ px_manager_constructed (GObject *object)
   if (!self->force_online) {
     self->network_monitor = g_network_monitor_get_default ();
     g_signal_connect_object (G_OBJECT (self->network_monitor), "network-changed", G_CALLBACK (px_manager_on_network_changed), self, 0);
-    px_manager_on_network_changed (self->network_monitor, g_network_monitor_get_network_available (self->network_monitor), self);
+
+    /* Expect to be online until network-changed is emitted */
+    self->wpad = FALSE;
+    self->online = TRUE;
   } else {
     px_manager_on_network_changed (NULL, TRUE, self);
   }
@@ -440,16 +440,14 @@ px_manager_pac_download (PxManager  *self,
  * px_manager_get_configuration:
  * @self: a px manager
  * @uri: PAC uri
- * @error: a #GError
  *
  * Get raw proxy configuration for gien @uri.
  *
  * Returns: (transfer full) (nullable): a newly created `GStrv` containing configuration data for @uri.
  */
 char **
-px_manager_get_configuration (PxManager  *self,
-                              GUri       *uri,
-                              GError    **error)
+px_manager_get_configuration (PxManager *self,
+                              GUri      *uri)
 {
   g_autoptr (GStrvBuilder) builder = g_strv_builder_new ();
 
@@ -481,20 +479,21 @@ px_manager_run_pac (PxPacRunner  *pacrunner,
   for (int idx = 0; idx < g_strv_length (proxies_split); idx++) {
     char *line = g_strstrip (proxies_split[idx]);
     g_auto (GStrv) word_split = g_strsplit (line, " ", -1);
-    g_autoptr (GUri) proxy_uri = NULL;
-    char *method;
-    char *server;
 
     /* Check for syntax "METHOD SERVER" */
     if (g_strv_length (word_split) == 2) {
+      g_autoptr (GUri) proxy_uri = NULL;
       g_autofree char *uri_string = NULL;
       g_autofree char *proxy_string = NULL;
+      g_autoptr (GUri) test_uri = NULL;
+      char *method;
+      char *server;
 
       method = word_split[0];
       server = word_split[1];
 
       uri_string = g_strconcat ("http://", server, NULL);
-      proxy_uri = g_uri_parse (uri_string, G_URI_FLAGS_PARSE_RELAXED, NULL);
+      proxy_uri = g_uri_parse (uri_string, G_URI_FLAGS_NONE, NULL);
       if (!proxy_uri)
         continue;
 
@@ -510,14 +509,32 @@ px_manager_run_pac (PxPacRunner  *pacrunner,
         proxy_string = g_strconcat ("socks://", server, NULL);
       }
 
-      px_strv_builder_add_proxy (builder, proxy_string);
-    } else {
-      /* Syntax not found, returning direct */
-      px_strv_builder_add_proxy (builder, "direct://");
+      if (proxy_string) {
+        test_uri = g_uri_parse (proxy_string, G_URI_FLAGS_NONE, NULL);
+
+        if (test_uri)
+          px_strv_builder_add_proxy (builder, proxy_string);
+      }
     }
   }
 }
 
+static gboolean
+px_manager_set_pac (PxManager *self)
+{
+  GList *list;
+
+  for (list = self->pacrunner_plugins; list && list->data; list = list->next) {
+    PxPacRunner *pacrunner = PX_PAC_RUNNER (list->data);
+    PxPacRunnerInterface *ifc = PX_PAC_RUNNER_GET_IFACE (pacrunner);
+
+    if (!ifc->set_pac (PX_PAC_RUNNER (pacrunner), self->pac_data))
+      return FALSE;
+  }
+
+  return TRUE;
+}
+
 static gboolean
 px_manager_expand_wpad (PxManager *self,
                         GUri      *uri)
@@ -535,7 +552,7 @@ px_manager_expand_wpad (PxManager *self,
     }
 
     if (!self->pac_data) {
-      GUri *wpad_url = g_uri_parse ("http://wpad/wpad.dat", G_URI_FLAGS_PARSE_RELAXED, NULL);
+      GUri *wpad_url = g_uri_parse ("http://wpad/wpad.dat", G_URI_FLAGS_NONE, NULL);
 
       g_debug ("%s: Trying to find the PAC using WPAD...", __FUNCTION__);
       self->pac_url = g_uri_to_string (wpad_url);
@@ -543,6 +560,14 @@ px_manager_expand_wpad (PxManager *self,
       if (!self->pac_data) {
         g_clear_pointer (&self->pac_url, g_free);
         ret = FALSE;
+      } else {
+        g_debug ("%s: PAC recevied!", __FUNCTION__);
+        if (!px_manager_set_pac (self)) {
+          g_warning ("%s: Unable to set PAC from %s while online = %d!", __FUNCTION__, self->pac_url, self->online);
+          g_clear_pointer (&self->pac_url, g_free);
+          g_clear_pointer (&self->pac_data, g_bytes_unref);
+          ret = FALSE;
+        }
       }
     }
   }
@@ -550,22 +575,6 @@ px_manager_expand_wpad (PxManager *self,
   return ret;
 }
 
-static gboolean
-px_manager_set_pac (PxManager *self)
-{
-  GList *list;
-
-  for (list = self->pacrunner_plugins; list && list->data; list = list->next) {
-    PxPacRunner *pacrunner = PX_PAC_RUNNER (list->data);
-    PxPacRunnerInterface *ifc = PX_PAC_RUNNER_GET_IFACE (pacrunner);
-
-    if (!ifc->set_pac (PX_PAC_RUNNER (pacrunner), self->pac_data))
-      return FALSE;
-  }
-
-  return TRUE;
-}
-
 static gboolean
 px_manager_expand_pac (PxManager *self,
                        GUri      *uri)
@@ -621,18 +630,18 @@ px_manager_expand_pac (PxManager *self,
  * Returns: (transfer full) (nullable): a newly created `GStrv` containing proxy related information.
  */
 char **
-px_manager_get_proxies_sync (PxManager   *self,
-                             const char  *url,
-                             GError     **error)
+px_manager_get_proxies_sync (PxManager  *self,
+                             const char *url)
 {
   g_autoptr (GStrvBuilder) builder = NULL;
   g_autoptr (GUri) uri = NULL;
   g_auto (GStrv) config = NULL;
+  g_autoptr (GError) error = NULL;
 
   g_mutex_lock (&self->mutex);
 
   builder = g_strv_builder_new ();
-  uri = g_uri_parse (url, G_URI_FLAGS_PARSE_RELAXED, error);
+  uri = g_uri_parse (url, G_URI_FLAGS_NONE, &error);
 
   g_debug ("%s: url=%s online=%d", __FUNCTION__, url ? url : "?", self->online);
   if (!uri || !self->online) {
@@ -641,10 +650,10 @@ px_manager_get_proxies_sync (PxManager   *self,
     return g_strv_builder_end (builder);
   }
 
-  config = px_manager_get_configuration (self, uri, error);
+  config = px_manager_get_configuration (self, uri);
 
   for (int idx = 0; idx < g_strv_length (config); idx++) {
-    GUri *conf_url = g_uri_parse (config[idx], G_URI_FLAGS_PARSE_RELAXED, NULL);
+    GUri *conf_url = g_uri_parse (config[idx], G_URI_FLAGS_NONE, NULL);
 
     g_debug ("%s: Config[%d] = %s", __FUNCTION__, idx, config[idx]);
 
@@ -697,6 +706,9 @@ ignore_domain (GUri *uri,
   if (g_strcmp0 (ignore, "*") == 0)
     return TRUE;
 
+  if (!host)
+    return FALSE;
+
   ignore_split = g_strsplit (ignore, ":", -1);
   port = g_uri_get_port (uri);
 
@@ -736,6 +748,9 @@ ignore_hostname (GUri *uri,
 {
   const char *host = g_uri_get_host (uri);
 
+  if (!host)
+    return FALSE;
+
   if (g_strcmp0 (ignore, "<local>") == 0 && strchr (host, ':') == NULL && strchr (host, '.') == NULL)
     return TRUE;
 
